@@ -14,20 +14,32 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
-package Dpkg::Source::Patch;
+=encoding utf8
+
+=head1 NAME
+
+Dpkg::Source::Patch - represent a patch file
+
+=head1 DESCRIPTION
+
+This module provides a class to handle patch files.
+
+B<Note>: This is a private module, its API can change at any time.
+
+=cut
+
+package Dpkg::Source::Patch 0.01;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.01';
-
 use POSIX qw(:errno_h :sys_wait_h);
 use File::Find;
 use File::Basename;
 use File::Spec;
 use File::Path qw(make_path);
 use File::Compare;
-use Fcntl ':mode';
+use Fcntl qw(:mode);
 use Time::HiRes qw(stat);
 
 use Dpkg;
@@ -62,6 +74,16 @@ sub set_header {
     *$self->{header} = $header;
 }
 
+sub get_header {
+    my $self = shift;
+
+    if (ref *$self->{header} eq 'CODE') {
+        return *$self->{header}->();
+    } else {
+        return *$self->{header};
+    }
+}
+
 sub add_diff_file {
     my ($self, $old, $new, %opts) = @_;
     $opts{include_timestamp} //= 0;
@@ -102,7 +124,7 @@ sub add_diff_file {
     my $diffgen;
     my $diff_pid = spawn(
         exec => [ 'diff', '-u', @options, '--', $old, $new ],
-        env => { LC_ALL => 'C', LANG => 'C', TZ => 'UTC0' },
+        env => { LC_ALL => 'C', TZ => 'UTC0' },
         to_pipe => \$diffgen,
     );
     # Check diff and write it in patch file
@@ -125,7 +147,7 @@ sub add_diff_file {
             error(g_("unknown line from diff -u on %s: '%s'"), $new, $_);
         }
 	if (*$self->{empty} and defined(*$self->{header})) {
-	    $self->print(*$self->{header}) or syserr(g_('failed to write'));
+	    $self->print($self->get_header()) or syserr(g_('failed to write'));
 	    *$self->{empty} = 0;
 	}
         print { $self } $_ or syserr(g_('failed to write'));
@@ -487,10 +509,18 @@ sub analyze {
 	    $dirtocreate{$dirname} = 1;
 	}
 
-	if (-e $fn and not -f _) {
-	    error(g_("diff '%s' patches something which is not a plain file"),
-	          $diff);
-	}
+        if (-e $fn) {
+            if (not -f _) {
+                error(g_("diff '%s' patches something which is not a plain file"),
+                      $diff);
+            }
+            # Note: We cannot use "stat _" due to Time::HiRes.
+            my $nlink = (stat $fn)[3];
+            if ($nlink > 1) {
+                warning(g_("diff '%s' patches hard link %s which can have " .
+                           'unintended consequences'), $diff, $fn);
+            }
+        }
 
 	if ($filepatched{$fn}) {
             $filepatched{$fn}++;
@@ -573,8 +603,16 @@ sub apply {
     $opts{force_timestamp} //= 1;
     $opts{remove_backup} //= 1;
     $opts{create_dirs} //= 1;
-    $opts{options} ||= [ '-t', '-F', '0', '-N', '-p1', '-u',
-            '-V', 'never', '-b', '-z', '.dpkg-orig'];
+    $opts{options} ||= [
+        '-t',
+        '-F', '0',
+        '-N',
+        '-p1',
+        '-u',
+        '-V', 'never',
+        '-b',
+        '-z', '.dpkg-orig',
+    ];
     $opts{add_options} //= [];
     push @{$opts{options}}, @{$opts{add_options}};
     # Check the diff and create missing directories
@@ -586,7 +624,7 @@ sub apply {
     spawn(
 	exec => [ $Dpkg::PROGPATCH, @{$opts{options}} ],
 	chdir => $destdir,
-	env => { LC_ALL => 'C', LANG => 'C', PATCH_GET => '0' },
+        env => { LC_ALL => 'C', PATCH_GET => '0' },
 	delete_env => [ 'POSIXLY_CORRECT' ], # ensure expected patch behaviour
 	wait_child => 1,
 	nocheck => 1,
@@ -624,8 +662,18 @@ sub check_apply {
     my ($self, $destdir, %opts) = @_;
     # Set default values to options
     $opts{create_dirs} //= 1;
-    $opts{options} ||= [ '--dry-run', '-s', '-t', '-F', '0', '-N', '-p1', '-u',
-            '-V', 'never', '-b', '-z', '.dpkg-orig'];
+    $opts{options} ||= [
+        '--dry-run',
+        '-s',
+        '-t',
+        '-F', '0',
+        '-N',
+        '-p1',
+        '-u',
+        '-V', 'never',
+        '-b',
+        '-z', '.dpkg-orig',
+    ];
     $opts{add_options} //= [];
     push @{$opts{options}}, @{$opts{add_options}};
     # Check the diff and create missing directories
@@ -636,7 +684,7 @@ sub check_apply {
     my $patch_pid = spawn(
 	exec => [ $Dpkg::PROGPATCH, @{$opts{options}} ],
 	chdir => $destdir,
-	env => { LC_ALL => 'C', LANG => 'C', PATCH_GET => '0' },
+        env => { LC_ALL => 'C', PATCH_GET => '0' },
 	delete_env => [ 'POSIXLY_CORRECT' ], # ensure expected patch behaviour
 	from_handle => $self->get_filehandle(),
 	to_file => '/dev/null',
@@ -666,4 +714,12 @@ sub get_type {
     }
 }
 
+=head1 CHANGES
+
+=head2 Version 0.xx
+
+This is a private module.
+
+=cut
+
 1;
