@@ -143,13 +143,13 @@ while (@ARGV) {
 }
 
 umask 0022; # ensure sane default permissions for created files
-my %options = (file => $changelogfile);
-$options{changelogformat} = $changelogformat if $changelogformat;
-my $changelog = changelog_parse(%options);
+my %changelog_opts = (file => $changelogfile);
+$changelog_opts{changelogformat} = $changelogformat if $changelogformat;
+my $changelog = changelog_parse(%changelog_opts);
 if ($changelog->{'Binary-Only'}) {
-    $options{count} = 1;
-    $options{offset} = 1;
-    my $prev_changelog = changelog_parse(%options);
+    $changelog_opts{count} = 1;
+    $changelog_opts{offset} = 1;
+    my $prev_changelog = changelog_parse(%changelog_opts);
     $sourceversion = $prev_changelog->{'Version'};
 } else {
     $sourceversion = $changelog->{'Version'};
@@ -232,6 +232,10 @@ foreach my $f (keys %{$pkg}) {
     }
 }
 
+if ((not debarch_eq('all', $fields->{'Architecture'})) && defined($ENV{DEB_HOST_ARCH_VARIANT})) {
+    $fields->{"Architecture-Variant"} = $ENV{DEB_HOST_ARCH_VARIANT};
+}
+
 # Scan fields of dpkg-parsechangelog
 foreach my $f (keys %{$changelog}) {
     my $v = $changelog->{$f};
@@ -312,6 +316,16 @@ for my $f (qw(Maintainer Description)) {
     warning(g_('missing information for output field %s'), $f)
         unless defined $fields->{$f};
 }
+for my $f (qw(Section)) {
+    next if defined $fields->{$f};
+
+    $fields->{$f} = field_get_default_value($f);
+    warning(g_('missing information for output field %s; ' .
+               'using default value "%s"'), $f, $fields->{$f});
+}
+for my $f (qw(Priority)) {
+    $fields->{$f} //= field_get_default_value($f);
+}
 
 my $pkg_type = $override{'Package-Type'} ||
                $pkg->{'Package-Type'} ||
@@ -389,9 +403,12 @@ if ($stdout) {
     my $sversion = $fields->{'Version'};
     $sversion =~ s/^\d+://;
     $forcefilename //= sprintf('%s_%s_%s.%s', $fields->{'Package'}, $sversion,
-                               $fields->{'Architecture'}, $pkg_type);
-    my $section = $fields->{'Section'} || '-';
-    my $priority = $fields->{'Priority'} || '-';
+                               $fields->{'Architecture-Variant'} || $fields->{'Architecture'}, $pkg_type);
+
+    my %fileprop;
+    foreach my $f (qw(Section Priority)) {
+        $fileprop{lc $f} = $fields->{$f};
+    }
 
     # Obtain a lock on debian/control to avoid simultaneous updates
     # of debian/files when parallel building is in use
@@ -411,6 +428,7 @@ if ($stdout) {
             ($file->{package} eq $fields->{'Package'}) &&
             ($file->{package_type} eq $pkg_type) &&
             (debarch_eq($file->{arch}, $fields->{'Architecture'}) ||
+             debarch_eq($file->{arch}, $fields->{'Architecture-Variant'}) ||
              debarch_eq($file->{arch}, 'all'))) {
             $dist->del_file($file->{filename});
         }
@@ -418,8 +436,9 @@ if ($stdout) {
 
     my %fileattrs;
     $fileattrs{automatic} = 'yes' if $fields->{'Auto-Built-Package'};
+    $fileattrs{architecture} = $fields->{'Architecture'};
 
-    $dist->add_file($forcefilename, $section, $priority, %fileattrs);
+    $dist->add_file($forcefilename, @fileprop{qw(section priority)}, %fileattrs);
     $dist->save("$fileslistfile.new");
 
     rename "$fileslistfile.new", $fileslistfile
